from unittest import TestCase

from identitylib.identifiers import Identifier, IdentifierSchemes, IdentifierScheme


class IdentifiersTestCase(TestCase):
    def test_identifiers_can_be_parsed(self):
        # standard parsing
        self.assertEqual(
            Identifier.from_string(
                "abc123@person.v1.student-records.university.identifiers.cam.ac.uk"
            ),
            Identifier("abc123", IdentifierSchemes.USN),
        )

        # fallback scheme given - actual scheme in identifier is used
        self.assertEqual(
            Identifier.from_string(
                "xxx@v1.person.identifiers.cam.ac.uk",
                fallback_scheme=IdentifierSchemes.USN,
            ),
            Identifier("xxx", IdentifierSchemes.CRSID),
        )

    def test_identifiers_parse_back_to_strings(self):
        # standard parsing
        self.assertEqual(
            str(Identifier("1000", IdentifierSchemes.STAFF_NUMBER)),
            "1000@person.v1.human-resources.university.identifiers.cam.ac.uk",
        )

        # fallback scheme given - actual scheme in identifier is used
        self.assertEqual(
            Identifier.from_string(
                "xxx@v1.person.identifiers.cam.ac.uk",
                fallback_scheme=IdentifierSchemes.USN,
            ),
            Identifier("xxx", IdentifierSchemes.CRSID),
        )

    def test_mifare_id_removes_padded_zeros(self):
        self.assertEqual(
            Identifier.from_string("0204011010", fallback_scheme=IdentifierSchemes.MIFARE_ID),
            Identifier("204011010", IdentifierSchemes.MIFARE_ID),
        )
        self.assertEqual(
            Identifier.from_string("00001", fallback_scheme=IdentifierSchemes.MIFARE_ID),
            Identifier("1", IdentifierSchemes.MIFARE_ID),
        )
        self.assertEqual(
            Identifier.from_string(
                "000121@mifare-identifier.v1.card.university.identifiers.cam.ac.uk"
            ),
            Identifier("121", IdentifierSchemes.MIFARE_ID),
        )
        self.assertEqual(
            Identifier.from_string("00000", fallback_scheme=IdentifierSchemes.MIFARE_ID),
            Identifier("0", IdentifierSchemes.MIFARE_ID),
        )

    def test_invalid_identifiers_are_rejected(self):
        with self.assertRaisesRegex(ValueError, "Invalid identifier scheme cam.ac.uk"):
            Identifier.from_string("123@cam.ac.uk")
        with self.assertRaisesRegex(ValueError, "Invalid identifier no-scheme"):
            Identifier.from_string("no-scheme")

    def test_identifiers_can_be_created_from_aliases(self):
        self.assertEqual(
            Identifier.from_string("VBE@barcode.identifiers.lib.cam.ac.uk", find_by_alias=True),
            Identifier("VBE", IdentifierSchemes.BARCODE),
        )

        # when we use find_by_alias this should throw
        with self.assertRaisesRegex(
            ValueError, "Invalid identifier scheme barcode.identifiers.lib.cam.ac.uk"
        ):
            Identifier.from_string("VBE@barcode.identifiers.lib.cam.ac.uk")

    def test_identifier_schemes_can_be_compared(self):
        self.assertTrue(
            IdentifierSchemes.PHOTO
            == IdentifierScheme(
                "photo.v1.photo.university.identifiers.cam.ac.uk",
                "Photo Identifier",
                {
                    "deprecated": "photo_id.photo.identifiers.uis.cam.ac.uk",
                },
            )
        )



# flake8: noqa

# import all models into this package
# if you have many models here with many references from one model to another this may
# raise a RecursionError
# to avoid this, import only the models that you directly need like:
# from identitylib.card_client.model.pet import Pet
# or import this package, but before doing it, use:
# import sys
# sys.setrecursionlimit(n)

from identitylib.card_client.model.api_exception import APIException
from identitylib.card_client.model.available_barcode import AvailableBarcode
from identitylib.card_client.model.available_barcode_batch_request_type import AvailableBarcodeBatchRequestType
from identitylib.card_client.model.available_barcode_batch_response_type import AvailableBarcodeBatchResponseType
from identitylib.card_client.model.available_barcode_batch_response_type_details import AvailableBarcodeBatchResponseTypeDetails
from identitylib.card_client.model.available_barcode_batch_response_type_details_invalid_inner import AvailableBarcodeBatchResponseTypeDetailsInvalidInner
from identitylib.card_client.model.available_barcode_create_request_type import AvailableBarcodeCreateRequestType
from identitylib.card_client.model.card import Card
from identitylib.card_client.model.card_filter_request_type import CardFilterRequestType
from identitylib.card_client.model.card_filter_response_type import CardFilterResponseType
from identitylib.card_client.model.card_identifier import CardIdentifier
from identitylib.card_client.model.card_identifier_bulk_update_request_type import CardIdentifierBulkUpdateRequestType
from identitylib.card_client.model.card_identifier_bulk_update_request_type_updates_inner import CardIdentifierBulkUpdateRequestTypeUpdatesInner
from identitylib.card_client.model.card_identifier_bulk_update_response_type import CardIdentifierBulkUpdateResponseType
from identitylib.card_client.model.card_identifier_bulk_update_response_type_details_inner import CardIdentifierBulkUpdateResponseTypeDetailsInner
from identitylib.card_client.model.card_identifier_destroy_response_type import CardIdentifierDestroyResponseType
from identitylib.card_client.model.card_identifier_update_request_type import CardIdentifierUpdateRequestType
from identitylib.card_client.model.card_identifier_update_response_type import CardIdentifierUpdateResponseType
from identitylib.card_client.model.card_logo import CardLogo
from identitylib.card_client.model.card_note import CardNote
from identitylib.card_client.model.card_note_create_request_type import CardNoteCreateRequestType
from identitylib.card_client.model.card_note_destroy_request_type import CardNoteDestroyRequestType
from identitylib.card_client.model.card_note_destroy_response_type import CardNoteDestroyResponseType
from identitylib.card_client.model.card_rfid_data_config_list_response_type import CardRFIDDataConfigListResponseType
from identitylib.card_client.model.card_rfid_data_config_list_response_type_results_inner import CardRFIDDataConfigListResponseTypeResultsInner
from identitylib.card_client.model.card_request import CardRequest
from identitylib.card_client.model.card_request_bulk_update_request_type import CardRequestBulkUpdateRequestType
from identitylib.card_client.model.card_request_bulk_update_request_type_updates_inner import CardRequestBulkUpdateRequestTypeUpdatesInner
from identitylib.card_client.model.card_request_bulk_update_request_type_updates_inner_fields import CardRequestBulkUpdateRequestTypeUpdatesInnerFields
from identitylib.card_client.model.card_request_bulk_update_request_type_updates_inner_identifiers_inner import CardRequestBulkUpdateRequestTypeUpdatesInnerIdentifiersInner
from identitylib.card_client.model.card_request_bulk_update_response_type import CardRequestBulkUpdateResponseType
from identitylib.card_client.model.card_request_create_request_type import CardRequestCreateRequestType
from identitylib.card_client.model.card_request_distinct_values import CardRequestDistinctValues
from identitylib.card_client.model.card_request_summary import CardRequestSummary
from identitylib.card_client.model.card_request_update_request_type import CardRequestUpdateRequestType
from identitylib.card_client.model.card_request_update_response_type import CardRequestUpdateResponseType
from identitylib.card_client.model.card_summary import CardSummary
from identitylib.card_client.model.card_update_request_type import CardUpdateRequestType
from identitylib.card_client.model.card_update_response_type import CardUpdateResponseType
from identitylib.card_client.model.college_institution_ids_list_response_type import CollegeInstitutionIdsListResponseType
from identitylib.card_client.model.paginated_available_barcode_type import PaginatedAvailableBarcodeType
from identitylib.card_client.model.paginated_card_identifier_type import PaginatedCardIdentifierType
from identitylib.card_client.model.paginated_card_logo_type import PaginatedCardLogoType
from identitylib.card_client.model.paginated_card_note_type import PaginatedCardNoteType
from identitylib.card_client.model.paginated_card_request_summary_type import PaginatedCardRequestSummaryType
from identitylib.card_client.model.paginated_card_summary_type import PaginatedCardSummaryType
from identitylib.card_client.model.permissions_response_type import PermissionsResponseType
from identitylib.card_client.model.validation_error import ValidationError
from identitylib.card_client.model.version_response_type import VersionResponseType




# flake8: noqa

# Import all APIs into this package.
# If you have many APIs here with many many models used in each API this may
# raise a `RecursionError`.
# In order to avoid this, import only the API that you directly need like:
#
#   from identitylib.photo_client.api.api_versions_api import APIVersionsApi
#
# or import this package, but before doing it, use:
#
#   import sys
#   sys.setrecursionlimit(n)

# Import APIs into API package:
from identitylib.photo_client.api.api_versions_api import APIVersionsApi
from identitylib.photo_client.api.all_photos_api import AllPhotosApi
from identitylib.photo_client.api.approved_photos_api import ApprovedPhotosApi
from identitylib.photo_client.api.permissions_api import PermissionsApi
from identitylib.photo_client.api.photo_identifier_api import PhotoIdentifierApi
from identitylib.photo_client.api.photo_identifiers_api import PhotoIdentifiersApi
from identitylib.photo_client.api.photo_identifiers_including_photos_api import PhotoIdentifiersIncludingPhotosApi
from identitylib.photo_client.api.photos_including_unapproved_photos_api import PhotosIncludingUnapprovedPhotosApi
from identitylib.photo_client.api.unapproved_photos_api import UnapprovedPhotosApi
